@@ -15,6 +15,7 @@ from osc.fetch import *
 from osc.core import get_buildinfo, store_read_apiurl, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig
 import osc.conf
 import oscerr
+import subprocess
 try:
     from xml.etree import cElementTree as ET
 except ImportError:
@@ -23,20 +24,29 @@ except ImportError:
 from conf import config
 
 change_personality = {
-            'i686': 'linux32',
-            'i586': 'linux32',
-            'i386': 'linux32',
-            'ppc': 'powerpc32',
-            's390': 's390',
+            'i686':  'linux32',
+            'i586':  'linux32',
+            'i386':  'linux32',
+            'ppc':   'powerpc32',
+            's390':  's390',
+        }
+
+change_personality_cross = {
+            'armv4l':   ' ',
+            'armv5el':  ' ',
+            'sh4':      ' ',
         }
 
 can_also_build = { 
-             'x86_64': ['i686', 'i586', 'i386'],
-             'i686': ['i586'],
-             'i386': ['i586'],
-             'ppc64': ['ppc'],
-             's390x': ['s390'],
-            }
+             'armv4l': [                                         'armv4l'                                 ],
+             'armv5el':[                                         'armv4l', 'armv5el'                      ],
+             's390x':  ['s390'                                                                            ],
+             'ppc64':  [                        'ppc', 'ppc64',                                           ],
+             'i386':   [        'i586',                          'armv4l', 'armv5el',               'sh4' ],
+             'i586':   [                'i386',                  'armv4l', 'armv5el',               'sh4' ],
+             'i686':   [        'i586',                          'armv4l', 'armv5el',               'sh4' ],
+             'x86_64': ['i686', 'i586', 'i386',                  'armv4l', 'armv5el',               'sh4' ],
+             }
 
 # real arch of this machine
 hostarch = os.uname()[4]
@@ -75,44 +85,33 @@ class Buildinfo:
         # are we building  .rpm or .deb?
         # need the right suffix for downloading
         # if a package named debhelper is in the dependencies, it must be .deb
+        # XXX: shouldn't we deliver the type via the buildinfo?
         self.pacsuffix = 'rpm'
-        for node in root.findall('dep'):
-            if node.text == 'debhelper':
+        for node in root.findall('bdep'):
+            if node.get('name') == 'debhelper':
                 self.pacsuffix = 'deb'
                 break
 
         self.buildarch = root.find('arch').text
+        if root.find('debuginfo') != None:
+            self.debuginfo = root.find('debuginfo').text
+        else:
+            self.debuginfo = 0
 
         self.deps = []
         for node in root.findall('bdep'):
-            p_name = node.get('name')
-            p_arch = node.get('arch')
-            if not p_arch:
-                p_arch = self.buildarch
-            p_version = node.get('version')
-            p_release = node.get('release')
-
-            if not (p_name and p_arch and p_version):
-                raise oscerr.APIError(
-                    "buildinfo for package %s/%s/%s is incomplete" % (p_name, p_arch, p_version))
-
-            p = Pac(p_name,
-                    p_version,
-                    p_release,
-                    node.get('project'),
-                    node.get('repository'),
-                    p_arch,
-                    node.get('preinstall'),
-                    node.get('runscripts'),
+            p = Pac(node,
                     self.buildarch,       # buildarch is used only for the URL to access the full tree...
                     self.pacsuffix,
                     scheme,
                     apisrv)
+                    
             self.deps.append(p)
 
         self.preinstall_list = [ dep.name for dep in self.deps if dep.preinstall ]
         self.runscripts_list = [ dep.name for dep in self.deps if dep.runscripts ]
 
+
     def has_dep(self, name):
         for i in self.deps:
             if i.name == name:
@@ -128,33 +127,34 @@ class Buildinfo:
 
 
 class Pac:
-    """represent a package to be downloaded"""
-    def __init__(self, name, version, release, project, repository, arch, 
-                 preinstall, runscripts, buildarch, pacsuffix, scheme=config['scheme'], apisrv=config['apisrv']):
+    """represent a package to be downloaded
 
-        self.name = name
-        self.version = version
-        self.release = release
-        self.arch = arch
-        self.project = project
-        self.repository = repository
-        self.preinstall = preinstall
-        self.runscripts = runscripts
-        self.buildarch = buildarch
-        self.pacsuffix = pacsuffix
+    We build a map that's later used to fill our URL templates
+    """
+    def __init__(self, node, 
+                 buildarch, pacsuffix, 
+                 scheme=config['scheme'], apisrv=config['apisrv']):
 
-        # build a map to fill our the URL templates
         self.mp = {}
-        self.mp['name'] = self.name
-        self.mp['version'] = self.version
-        self.mp['release'] = self.release
-        self.mp['arch'] = self.arch
-        self.mp['project'] = self.project
-        self.mp['repository'] = self.repository
-        self.mp['preinstall'] = self.preinstall
-        self.mp['runscripts'] = self.runscripts
-        self.mp['buildarch'] = self.buildarch
-        self.mp['pacsuffix'] = self.pacsuffix
+        for i in ['name', 'package', 
+                  'version', 'release', 
+                  'project', 'repository', 
+                  'preinstall', 'noinstall', 'runscripts',
+                 ]:
+            self.mp[i] = node.get(i)
+
+        self.mp['buildarch']  = buildarch
+        self.mp['pacsuffix']  = pacsuffix
+
+
+        self.mp['repopackage'] = node.get('repopackage') or '_repository'
+        self.mp['arch'] = node.get('arch') or self.mp['buildarch']
+        self.mp['repoarch'] = node.get('repoarch') or self.mp['arch']
+
+        if not (self.mp['name'] and self.mp['arch'] and self.mp['version']):
+            raise oscerr.APIError(
+                "buildinfo for package %s/%s/%s is incomplete" 
+                    % (self.mp['name'], self.mp['arch'], self.mp['version']))
 
         self.mp['scheme'] = scheme
         self.mp['apisrv'] = apisrv
@@ -163,6 +163,9 @@ class Pac:
 
         self.mp['filename'] = self.filename
 
+        # make the content of the dictionary accessible as class attributes
+        self.__dict__.update(self.mp)
+
 
     def makeurls(self, cachedir, urllist):
 
@@ -194,13 +197,16 @@ class Pac:
 
 def get_built_files(pacdir, pactype):
     if pactype == 'rpm':
-        b_built = os.popen('find %s -name \*.rpm' \
-                    % os.path.join(pacdir, 'RPMS')).read().strip()
-        s_built = os.popen('find %s -name \*.rpm' \
-                    % os.path.join(pacdir, 'SRPMS')).read().strip()
+        b_built = subprocess.Popen(['find', os.path.join(pacdir, 'RPMS'), 
+                                    '-name', '*.rpm'],
+                                   stdout=subprocess.PIPE).stdout.read().strip()
+        s_built = subprocess.Popen(['find', os.path.join(pacdir, 'SRPMS'), 
+                                    '-name', '*.rpm'],
+                                   stdout=subprocess.PIPE).stdout.read().strip()
     else:
-        b_built = os.popen('find %s -name \*.deb' \
-                    % os.path.join(pacdir, 'DEBS')).read().strip()
+        b_built = subprocess.Popen(['find', os.path.join(pacdir, 'DEBS'),
+                                    '-name', '*.deb'],
+                                   stdout=subprocess.PIPE).stdout.read().strip()
         s_built = None
     return s_built, b_built
 
@@ -218,9 +224,11 @@ def get_prefer_pkgs(dirs, wanted_arch):
             continue
         if path.find('-debuginfo-') > 0:
             continue
-        arch, name = os.popen('rpm -qp --nosignature --nodigest --qf "%%{arch} %%{name}\\n" %s' \
-                       % path).read().split()
-        # instead of this assumption, we should probably rather take the
+        arch, name = subprocess.Popen(['rpm', '-qp', 
+                                      '--nosignature', '--nodigest', 
+                                      '--qf', '%{arch} %{name}\n', path], 
+                                      stdout=subprocess.PIPE).stdout.read().split()
+        # instead of thip assumption, we should probably rather take the
         # requested arch for this package from buildinfo
         # also, it will ignore i686 packages, how to handle those?
         if arch == wanted_arch or arch == 'noarch':
@@ -233,7 +241,14 @@ def main(opts, argv):
 
     repo = argv[0]
     arch = argv[1]
-    spec = argv[2]
+    build_descr = argv[2]
+    crossbuild = False
+
+    build_type = os.path.splitext(build_descr)[1][1:]
+    if build_type not in ['spec', 'dsc', 'kiwi']:
+        raise oscerr.WrongArgs(
+                "Unknown build type: '%s'. Build description should end in .spec, .dsc or .kiwi." \
+                        % build_type)
 
     buildargs = []
     if not opts.userootforbuild:
@@ -242,40 +257,32 @@ def main(opts, argv):
         buildargs.append('--clean')
     if opts.noinit:
         buildargs.append('--noinit')
+    if opts.nochecks:
+        buildargs.append('--no-checks')
     if not opts.no_changelog:
         buildargs.append('--changelog')
     if opts.jobs:
         buildargs.append('--jobs %s' % opts.jobs)
     if opts.baselibs:
         buildargs.append('--baselibs')
-    buildargs = ' '.join(buildargs)
+    if opts.debuginfo:
+        buildargs.append('--debug')
 
-    prj = store_read_project(os.curdir)
-    pac = store_read_package(os.curdir)
     if opts.local_package:
         pac = '_repository'
     if opts.alternative_project:
         prj = opts.alternative_project
         pac = '_repository'
+        apiurl = config['apiurl']
+    else:
+        prj = store_read_project(os.curdir)
+        pac = store_read_package(os.curdir)
+        apiurl = store_read_apiurl(os.curdir)
 
-    if not os.path.exists(spec):
-        print >>sys.stderr, 'Error: specfile \'%s\' does not exist.' % spec
+    if not os.path.exists(build_descr):
+        print >>sys.stderr, 'Error: build description named \'%s\' does not exist.' % build_descr
         return 1
 
-    if opts.debuginfo:
-        # make sure %debug_package is in the spec-file.
-        spec_text = open(spec).read()
-        if not re.search(r'(?m)^%debug_package', spec_text):
-            spec_text = re.sub(r'(?m)^(%prep)', 
-                r'# added by osc build -d\n%debug_package\n\n\1', 
-                spec_text, 1)
-            tmp_spec = NamedTemporaryFile(prefix = spec + '_', dir = '.', suffix = '.spec')
-            tmp_spec.write(spec_text)
-            tmp_spec.flush()
-            spec = tmp_spec.name
-            os.chmod(spec, 0644)
-
-
     # make it possible to override configuration of the rc file
     for var in ['OSC_PACKAGECACHEDIR', 'OSC_SU_WRAPPER', 'OSC_BUILD_ROOT']: 
         val = os.getenv(var)
@@ -298,20 +305,20 @@ def main(opts, argv):
     print 'Getting buildinfo from server'
     bi_file = NamedTemporaryFile(suffix='.xml', prefix='buildinfo.', dir = '/tmp')
     try:
-        bi_text = ''.join(get_buildinfo(store_read_apiurl(os.curdir), 
+        bi_text = ''.join(get_buildinfo(apiurl, 
                                         prj,
                                         pac,
                                         repo, 
                                         arch, 
-                                        specfile=open(spec).read(), 
+                                        specfile=open(build_descr).read(), 
                                         addlist=extra_pkgs))
     except urllib2.HTTPError, e:
         if e.code == 404:
         # check what caused the 404
             if meta_exists(metatype='prj', path_args=(quote_plus(prj), ),
                            template_args=None, create_new=False):
-                if meta_exists(metatype='pkg', path_args=(quote_plus(prj), quote_plus(pac)),
-                               template_args=None, create_new=False) or pac == '_repository':
+                if pac == '_repository' or meta_exists(metatype='pkg', path_args=(quote_plus(prj), quote_plus(pac)),
+                                                       template_args=None, create_new=False):
                     print >>sys.stderr, 'wrong repo/arch?'
                     sys.exit(1)
                 else:
@@ -327,7 +334,18 @@ def main(opts, argv):
     bi_file.write(bi_text)
     bi_file.flush()
 
-    bi = Buildinfo(bi_file.name, store_read_apiurl(os.curdir))
+    bi = Buildinfo(bi_file.name, apiurl)
+    if bi.debuginfo:
+        buildargs.append('--debug')
+    buildargs = ' '.join(set(buildargs))
+
+    # real arch of this machine 
+    # vs.
+    # arch we are supposed to build for
+    if hostarch != bi.buildarch:
+        if not bi.buildarch in can_also_build.get(hostarch, []):
+            print >>sys.stderr, 'Error: hostarch \'%s\' cannot build \'%s\'.' % (hostarch, bi.buildarch)
+            return 1
 
     rpmlist_prefers = []
     if opts.prefer_pkgs:
@@ -351,12 +369,15 @@ def main(opts, argv):
     print 'Updating cache of required packages'
     fetcher = Fetcher(cachedir = config['packagecachedir'], 
                       urllist = config['urllist'],
-                      auth_dict = config['auth_dict'],
+                      api_host_options = config['api_host_options'],
                       http_debug = config['http_debug'])
 
     # now update the package cache
     fetcher.run(bi)
 
+    if build_type == 'kiwi' and not os.path.exists('repos'):
+        os.symlink(config['packagecachedir'], 'repos')
+
     if bi.pacsuffix == 'rpm':
         """don't know how to verify .deb packages. They are verified on install
         anyway, I assume... verifying package now saves time though, since we don't
@@ -370,7 +391,7 @@ def main(opts, argv):
 
     print 'Writing build configuration'
 
-    rpmlist = [ '%s %s\n' % (i.name, i.fullfilename) for i in bi.deps ]
+    rpmlist = [ '%s %s\n' % (i.name, i.fullfilename) for i in bi.deps if not i.noinstall ]
     rpmlist += [ '%s %s\n' % (i[0], i[1]) for i in rpmlist_prefers ]
 
     rpmlist.append('preinstall: ' + ' '.join(bi.preinstall_list) + '\n')
@@ -385,40 +406,53 @@ def main(opts, argv):
 
     print 'Getting buildconfig from server'
     bc_file = NamedTemporaryFile(prefix='buildconfig.', dir = '/tmp')
-    bc_file.write(get_buildconfig(store_read_apiurl(os.curdir), prj, pac, repo, arch))
+    bc_file.write(get_buildconfig(apiurl, prj, pac, repo, arch))
     bc_file.flush()
 
 
     print 'Running build'
 
+    # check for cross-build
+    if hostarch == 'x86_64':
+       if bi.buildarch == 'armv4l':
+          crossbuild = True
+       if bi.buildarch == 'armv5el':
+          crossbuild = True
+       if bi.buildarch == 'sh4':
+          crossbuild = True
+
+    if hostarch == 'i586':
+       if bi.buildarch == 'armv4l':
+          crossbuild = True
+       if bi.buildarch == 'armv5el':
+          crossbuild = True
+       if bi.buildarch == 'sh4':
+          crossbuild = True
+
     cmd = '%s --root=%s --rpmlist=%s --dist=%s %s %s' \
                  % (config['build-cmd'],
                     config['build-root'],
                     rpmlist_file.name, 
                     bc_file.name, 
-                    spec, 
+                    build_descr, 
                     buildargs)
 
     if config['su-wrapper'].startswith('su '):
         tmpl = '%s \'%s\''
     else:
         tmpl = '%s %s'
-    cmd = tmpl % (config['su-wrapper'], cmd)
-        
-    # real arch of this machine 
-    # vs.
-    # arch we are supposed to build for
-    if hostarch != bi.buildarch:
 
-        # change personality, if needed
-        if bi.buildarch in can_also_build.get(hostarch, []):
-            cmd = change_personality[bi.buildarch] + ' ' + cmd
-        else:
-            print >>sys.stderr, 'Error: hostarch \'%s\' cannot build \'%s\'.' % (hostarch, bi.buildarch)
-            return 1
+    # change personality, if needed
+    cmd = tmpl % (config['su-wrapper'], cmd)
+    if crossbuild == False:
+       if hostarch != bi.buildarch:
+          cmd = change_personality[bi.buildarch] + ' ' + cmd
+
+    if crossbuild == True:
+          cmd = change_personality_cross[bi.buildarch] + ' ' + cmd + ' --arch=%s' % (bi.buildarch)
 
     print cmd
-    rc = os.system(cmd)
+    rc = subprocess.call(cmd, shell=True)
     if rc: 
         print
         print 'The buildroot was:', config['build-root']
