@@ -271,8 +271,8 @@ class RawCmdln(cmd.Cmd):
 
         Returns the return value from the command handler.
         """
-        assert (isinstance(argv, (list, tuple)), 
-                "'argv' is not a sequence: %r" % argv)
+        assert isinstance(argv, (list, tuple)), \
+                "'argv' is not a sequence: %r" % argv
         retval = None
         try:
             argv = self.precmd(argv)
@@ -317,8 +317,8 @@ class RawCmdln(cmd.Cmd):
         while not self.stop:
             if self.cmdqueue:
                 argv = self.cmdqueue.pop(0)
-                assert (isinstance(argv, (list, tuple)), 
-                        "item on 'cmdqueue' is not a sequence: %r" % argv)
+                assert isinstance(argv, (list, tuple)), \
+                        "item on 'cmdqueue' is not a sequence: %r" % argv
             else:
                 if self.use_rawinput:
                     try:
