@@ -26,6 +26,9 @@ class Osc(cmdln.Cmdln):
     For additional information, see 
     * http://www.opensuse.org/Build_Service_Tutorial
     * http://www.opensuse.org/Build_Service/CLI
+
+    You can modify osc commands, or roll you own, via the plugin API:
+    * http://www.opensuse.org/Build_Service/osc_plugins
     """
     name = 'osc'
     conf = None
@@ -59,7 +62,7 @@ class Osc(cmdln.Cmdln):
         return optparser
 
 
-    def postoptparse(self):
+    def postoptparse(self, try_again = True):
         """merge commandline options into the config"""
         try:
             conf.get_config(override_conffile = self.options.conffile,
@@ -76,17 +79,23 @@ class Osc(cmdln.Cmdln):
             config['user'] = raw_input('Username: ')
             config['pass'] = getpass.getpass()
 
-            if conf.write_config(e.file, config):
-                print >>sys.stderr, 'done'
-                conf.get_config(override_conffile = self.options.conffile,
-                                override_apisrv = self.options.apisrv,
-                                override_debug = self.options.debug,
-                                override_http_debug = self.options.http_debug,
-                                override_traceback = self.options.traceback,
-                                override_post_mortem = self.options.post_mortem)
-            else:
-                raise NoConfigfile(e.file, 'Unable to create osc\'s configuration file \
-                                                 \'%s\'' % e.file)
+            conf.write_initial_config(e.file, config)
+            print >>sys.stderr, 'done'
+            if try_again: self.postoptparse(try_again = False)
+        except oscerr.ConfigMissingApiurl, e:
+            print >>sys.stderr, e.msg
+            import getpass
+            user = raw_input('Username: ')
+            passwd = getpass.getpass()
+            cp = conf.get_configParser()
+            cp.add_section(e.url)
+            cp.set(e.url, 'user', user)
+            cp.set(e.url, 'pass', passwd)
+            file = open(e.file, 'w')
+            cp.write(file, True)
+            if file: file.close()
+            if try_again: self.postoptparse(try_again = False)
+
         self.conf = conf
 
 
@@ -98,11 +107,11 @@ class Osc(cmdln.Cmdln):
         return self._str(doc)
 
 
-    def do_init(self, subcmd, opts, project, package):
+    def do_init(self, subcmd, opts, project, package=None):
         """${cmd_name}: Initialize a directory as working copy 
 
         Initialize an existing directory to be a working copy of an
-        (already existing) buildservice package. 
+        (already existing) buildservice project/package. 
 
         (This is the same as checking out a package and then copying sources
         into the directory. It does NOT create a new package. To create a
@@ -115,12 +124,17 @@ class Osc(cmdln.Cmdln):
         checkout" to get help for it.
         
         usage: 
+            osc init PRJ
             osc init PRJ PAC
         ${cmd_option_list}
         """
-
-        init_package_dir(conf.config['apiurl'], project, package, os.path.curdir)
-        print 'Initializing %s (Project: %s, Package: %s)' % (os.curdir, project, package)
+        
+        if not package:
+            init_project_dir(conf.config['apiurl'], os.curdir, project)
+            print 'Initializing %s (Project: %s)' % (os.curdir, project)
+        else:
+            init_package_dir(conf.config['apiurl'], project, package, os.path.curdir)
+            print 'Initializing %s (Project: %s, Package: %s)' % (os.curdir, project, package)
 
 
     @cmdln.alias('ls')
@@ -144,6 +158,7 @@ class Osc(cmdln.Cmdln):
            ls Apache                  # list packages in a project
            ls -b Apache               # list all binaries of a project
            ls Apache apache2          # list source files of package of a project
+           ls Apache apache2 <file>   # list <file> if this file exists
            ls -v Apache apache2       # verbosely list source files of package 
 
         With --verbose, the following fields will be shown for each item:
@@ -160,9 +175,14 @@ class Osc(cmdln.Cmdln):
 
         if len(args) == 1:
             project = args[0]
-        elif len(args) == 2:
+        elif len(args) == 2 or len(args) == 3:
             project = args[0]
             package = args[1]
+            fname = None
+            if len(args) == 3:
+                fname = args[2]
+        elif len(args) > 3:
+            raise oscerr.WrongArgs('Too many arguments')
 
         if opts.binaries and (not opts.repo or not opts.arch):
             raise oscerr.WrongOptions('Sorry, -r <repo> -a <arch> missing\n'
@@ -194,17 +214,27 @@ class Osc(cmdln.Cmdln):
 
                 print '\n'.join(meta_get_packagelist(conf.config['apiurl'], project))
 
-            elif len(args) == 2:
+            elif len(args) == 2 or len(args) == 3:
                 l = meta_get_filelist(conf.config['apiurl'], 
                                       project, 
                                       package,
                                       verbose=opts.verbose)
                 if opts.verbose:
-                    for i in l:
-                        print '%s %7d %9d %s %s' \
-                            % (i.md5, i.rev, i.size, shorttime(i.mtime), i.name)
+                    out = [ '%s %7d %9d %s %s' % (i.md5, i.rev, i.size, shorttime(i.mtime), i.name) \
+                            for i in l if not fname or fname == i.name ]
+                    if len(out) == 0:
+                        if fname:
+                            print 'file \'%s\' does not exist' % fname
+                    else:
+                        print '\n'.join(out)
                 else:
-                    print '\n'.join(l)
+                    if fname:
+                        if fname in l:
+                            print fname
+                        else:    
+                            print 'file \'%s\' does not exist' % fname
+                    else:
+                        print '\n'.join(l)
 
 
     @cmdln.option('-F', '--file', metavar='FILE',
@@ -406,7 +436,10 @@ class Osc(cmdln.Cmdln):
     @cmdln.option('--nodevelproject', action='store_true',
                   help='do not follow a defined devel project ' \
                        '(primary project where a package is developed)')
+    @cmdln.option('-s', '--state', default='new',
+                        help='only list requests in one of the comma separated given states [default=new]')
     @cmdln.alias("sr")
+    @cmdln.alias("submitrequest")
     def do_submitreq(self, subcmd, opts, *args):
         """${cmd_name}: Handle requests to submit a package into another project
 
@@ -426,13 +459,18 @@ class Osc(cmdln.Cmdln):
 
         "list" lists open requests attached to a project or package.
 
+        "log" will show the history of the given ID
+
         "show" will show the request itself, and generate a diff for review, if
         used with the --diff option.
 
         "decline" will change the request state to "declined" and append a
         message that you specify with the --message option.
 
-        "delete" will change the request state to "deleted" and append a
+        "delete" will permanently delete a request and append a
+        message that you specify with the --message option.
+
+        "revoke" will set the request state to "revoked" and append a
         message that you specify with the --message option.
 
         "accept" will change the request state to "accepted" and will trigger
@@ -444,16 +482,18 @@ class Osc(cmdln.Cmdln):
             osc submitreq create [-m TEXT] 
             osc submitreq create [-m TEXT] SOURCEPRJ SOURCEPKG DESTPRJ [DESTPKG]
             osc submitreq list [PRJ [PKG]]
+            osc submitreq log ID
             osc submitreq show [-d] ID
+            osc submitreq accept [-m TEXT] ID
             osc submitreq decline [-m TEXT] ID
             osc submitreq delete [-m TEXT] ID
-            osc submitreq accept [-m TEXT] ID
+            osc submitreq revoke [-m TEXT] ID
         ${cmd_option_list}
         """
 
         args = slash_split(args)
 
-        cmds = ['create', 'list', 'show', 'decline', 'accept', 'delete']
+        cmds = ['create', 'list', 'log', 'show', 'decline', 'accept', 'delete', 'revoke']
         if not args or args[0] not in cmds:
             raise oscerr.WrongArgs('Unknown submitreq action. Choose one of %s.' \
                                                % ', '.join(cmds))
@@ -515,41 +555,69 @@ class Osc(cmdln.Cmdln):
 
             if len(args) > 1:
                 package = args[1]
-        elif cmd in ['show', 'decline', 'accept', 'delete']:
+        elif cmd in ['log', 'show', 'decline', 'accept', 'delete', 'revoke']:
             reqid = args[0]
 
 
         # create
         if cmd == 'create':
-            devloc = show_develproject(apiurl, dst_project, dst_package)
-            if devloc \
-                    and dst_project != devloc \
-                    and src_project != devloc \
-                    and not opts.nodevelproject:
-                print """\
+            if not opts.nodevelproject:
+                devloc = None
+                try:
+                    devloc = show_develproject(apiurl, dst_project, dst_package)
+                except urllib2.HTTPError:
+                    print >>sys.stderr, """\
+Warning: failed to fetch meta data for '%s' package '%s' (new package?) """ \
+                        % (dst_project, dst_package)
+                    pass
+
+                if devloc \
+                      and dst_project != devloc \
+                      and src_project != devloc:
+                          print """\
 Sorry, but a different project, %s, is defined as the place where development
 of the package %s primarily takes place.
 Please submit there instead, or use --nodevelproject to force direct submission.""" \
                     % (devloc, dst_package)
-                sys.exit(1)
+                          sys.exit(1)
+            reqs = get_submit_request_list(apiurl, dst_project, dst_package)
+            user = conf.get_apiurl_usr(apiurl)
+            myreqs = [ i for i in reqs if i.state.who == user ]
+            repl = ''
+            if len(myreqs) > 0:
+                print 'You already created the following submitrequests: %s.' % \
+                      ', '.join([str(i.reqid) for i in myreqs ])
+                repl = raw_input('Revoke the old requests? (y/N) ')
 
             result = create_submit_request(apiurl, 
                                           src_project, src_package,
                                           dst_project, dst_package,
                                           opts.message, orev=opts.revision)
+            if repl == 'y':
+                for req in myreqs:
+                    change_submit_request_state(apiurl, str(req.reqid), 'revoked',
+                                                'superseeded by %s' % result)
+
             print 'created request id', result
 
 
         # list
         elif cmd == 'list':
-            results = get_submit_request_list(apiurl, 
-                                             project, package)
+            state_list = opts.state.split(',')
+
+            results = get_submit_request_list(apiurl,
+                                             project, package, state_list)
 
             results.sort(reverse=True)
 
             for result in results:
                 print result.list_view()
 
+        elif cmd == 'log':
+            for l in get_submit_request_log(conf.config['apiurl'], reqid):
+                print l
+
+
         # show
         elif cmd == 'show':
             r = get_submit_request(conf.config['apiurl'], reqid)
@@ -581,7 +649,11 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             r = change_submit_request_state(conf.config['apiurl'],
                     reqid, 'deleted', opts.message or '')
             print r
-
+        # revoke
+        elif cmd == 'revoke':
+            r = change_submit_request_state(conf.config['apiurl'],
+                    reqid, 'revoked', opts.message or '')
+            print r
 
     # editmeta and its aliases are all depracated
     @cmdln.alias("editprj")
@@ -657,6 +729,14 @@ Please submit there instead, or use --nodevelproject to force direct submission.
     def do_aggregatepac(self, subcmd, opts, *args):
         """${cmd_name}: "Aggregate" a package to another package
         
+        Aggregation of a package means that the build results (binaries) of a
+        package are basically copied into another project.
+        This can be used to make packages available from building that are
+        needed in a project but available only in a different project. Note
+        that this is done at the expense of disk space. See 
+        http://en.opensuse.org/Build_Service/Tips_and_Tricks#_link_and__aggregate
+        for more information.
+
         The DESTPAC name is optional; the source packages' name will be used if
         DESTPAC is omitted.
 
@@ -748,7 +828,7 @@ Please submit there instead, or use --nodevelproject to force direct submission.
     @cmdln.option('--nodevelproject', action='store_true',
                         help='do not follow a defined devel project ' \
                              '(primary project where a package is developed)')
-    def do_branch(self, subcmd, opts, prj, pkg):
+    def do_branch(self, subcmd, opts, *args):
         """${cmd_name}: Branch a package
 
         [See http://en.opensuse.org/Build_Service/Collaboration for information
@@ -765,9 +845,13 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         ${cmd_option_list}
         """
 
-        r = branch_pkg(conf.config['apiurl'], prj, pkg, nodevelproject=opts.nodevelproject)
+        args = slash_split(args)
+        if len(args) != 2:
+            raise oscerr.WrongArgs('Wrong number of arguments.')
 
-        expected = 'home:%s:branches:%s' % (conf.config['user'], prj)
+        r = branch_pkg(conf.config['apiurl'], args[0], args[1], nodevelproject=opts.nodevelproject)
+
+        expected = 'home:%s:branches:%s' % (conf.config['user'], args[0])
         if r != expected:
             devloc = r.split('branches:')[1]
             print '\nNote: The branch has been created of a different project,\n' \
@@ -780,10 +864,10 @@ Please submit there instead, or use --nodevelproject to force direct submission.
 
         print 'A working copy of the branched package can be checked out with:\n\n' \
               'osc co %s/%s' \
-                      % (r, pkg)
+                      % (r, args[1])
 
 
-    def do_deletepac(self, subcmd, opts, project, *pkgs):
+    def do_deletepac(self, subcmd, opts, *args):
         """${cmd_name}: Delete packages on the repository server
 
         usage:
@@ -791,11 +875,13 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         ${cmd_option_list}
         """
 
+        args = slash_split(args)
+        pkgs = args[1:]
         if not pkgs:
             raise oscerr.WrongArgs('Missing argument.')
 
         for pkg in pkgs:
-            delete_package(conf.config['apiurl'], project, pkg)
+            delete_package(conf.config['apiurl'], args[0], pkg)
 
 
     @cmdln.option('-f', '--force', action='store_true',
@@ -821,6 +907,7 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             delete_project(conf.config['apiurl'], project)
 
 
+    @cmdln.alias('metafromspec')
     @cmdln.option('', '--specfile', metavar='FILE',
                       help='Path to specfile. (if you pass more than working copy this option is ignored)')
     def do_updatepacmetafromspec(self, subcmd, opts, *args):
@@ -1002,8 +1089,15 @@ Please submit there instead, or use --nodevelproject to force direct submission.
 
             # all packages
             for package in meta_get_packagelist(conf.config['apiurl'], project):
-                checkout_package(conf.config['apiurl'], project, package, 
-                                 expand_link=expand_link, prj_dir=project)
+                try:
+                    checkout_package(conf.config['apiurl'], project, package, 
+                                     expand_link=expand_link, prj_dir=project)
+                except oscerr.LinkExpandError, e:
+                    print >>sys.stderr, 'Link cannot be expanded:\n', e
+                    # check out in unexpanded form at least
+                    checkout_package(conf.config['apiurl'], project, package, 
+                                     expand_link=False, prj_dir=project)
+
         else:
             raise oscerr.WrongArgs('Missing argument.\n\n' \
                   + self.get_cmd_help('checkout'))
@@ -1089,7 +1183,6 @@ Please submit there instead, or use --nodevelproject to force direct submission.
                   + self.get_cmd_help('add'))
 
         filenames = parseargs(args)
-        #print filenames
         addFiles(filenames)
 
     
@@ -1109,7 +1202,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
 
         createPackageDir(args[0])
 
-
+    @cmdln.option('-r', '--recursive', action='store_true',
+                        help='If CWD is a project dir then scan all package dirs as well')
     def do_addremove(self, subcmd, opts, *args):
         """${cmd_name}: Adds new files, removes disappeared files
 
@@ -1122,6 +1216,27 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         """
 
         args = parseargs(args)
+        arg_list = args[:]
+        for arg in arg_list:
+            if is_project_dir(arg):
+                prj = Project(arg, False)
+                for pac in prj.pacs_unvers:
+                    pac_dir = getTransActPath(os.path.join(prj.dir, pac))
+                    if os.path.isdir(pac_dir):
+                        addFiles([pac_dir], prj)
+                for pac in prj.pacs_broken:
+                    if prj.get_state(pac) != 'D':
+                        prj.set_state(pac, 'D')
+                        print statfrmt('D', getTransActPath(os.path.join(prj.dir, pac)))
+                if opts.recursive:
+                    for pac in prj.pacs_have:
+                        state = prj.get_state(pac)
+                        if state != None and state != 'D':
+                            pac_dir = getTransActPath(os.path.join(prj.dir, pac))
+                            args.append(pac_dir)
+                args.remove(arg)
+                prj.write_packages()
+
         pacs = findpacs(args)
         for p in pacs:
 
@@ -1136,12 +1251,12 @@ Please submit there instead, or use --nodevelproject to force direct submission.
                 state = p.status(filename)
                 if state == '?':
                     p.addfile(filename)
-                    print statfrmt('A', filename)
+                    print statfrmt('A', getTransActPath(os.path.join(p.dir, filename)))
                 elif state == '!':
                     p.put_on_deletelist(filename)
                     p.write_deletelist()
                     os.unlink(os.path.join(p.storedir, filename))
-                    print statfrmt('D', filename)
+                    print statfrmt('D', getTransActPath(os.path.join(p.dir, filename)))
 
 
 
@@ -1177,7 +1292,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             except:
                 sys.exit('could not open file \'%s\'.' % opts.file)
 
-        for arg in args:
+        arg_list = args[:]
+        for arg in arg_list:
             if conf.config['do_package_tracking'] and is_project_dir(arg):
                 Project(arg).commit(msg=msg)
                 args.remove(arg)
@@ -1257,8 +1373,9 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         """
 
         args = parseargs(args)
+        arg_list = args[:]
 
-        for arg in args:
+        for arg in arg_list:
 
             if is_project_dir(arg):
                 prj = Project(arg)
@@ -1297,15 +1414,16 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             if len(pacs) > 1:
                 print 'Updating %s' % p.name
 
-            if opts.expand_link and p.islink() and not p.isexpanded():
-                print 'Expanding to rev', p.linkinfo.xsrcmd5
-                rev = p.linkinfo.xsrcmd5
-            elif opts.unexpand_link and p.islink() and p.isexpanded():
-                print 'Unexpanding to rev', p.linkinfo.lsrcmd5
-                rev = p.linkinfo.lsrcmd5
-            elif p.islink() and p.isexpanded():
-                rev = show_upstream_xsrcmd5(p.apiurl,
-                                            p.prjname, p.name)
+            if not rev:
+                if opts.expand_link and p.islink() and not p.isexpanded():
+                    print 'Expanding to rev', p.linkinfo.xsrcmd5
+                    rev = p.linkinfo.xsrcmd5
+                elif opts.unexpand_link and p.islink() and p.isexpanded():
+                    print 'Unexpanding to rev', p.linkinfo.lsrcmd5
+                    rev = p.linkinfo.lsrcmd5
+                elif p.islink() and p.isexpanded():
+                    rev = show_upstream_xsrcmd5(p.apiurl,
+                                                p.prjname, p.name)
 
             # FIXME: ugly workaround for #399247
             if opts.expand_link or opts.unexpand_link:
@@ -1338,7 +1456,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         args = parseargs(args)
         # check if args contains a package which was removed by
         # a non-osc command and mark it with the 'D'-state
-        for i in args:
+        arg_list = args[:]
+        for i in arg_list:
             if not os.path.exists(i):
                 prj_dir, pac_dir = getPrjPacPaths(i)
                 if is_project_dir(prj_dir):
@@ -1481,8 +1600,10 @@ Please submit there instead, or use --nodevelproject to force direct submission.
     @cmdln.option('-s', '--status-filter', metavar='STATUS',
                         help='show only packages with buildstatus STATUS (see legend)')
     @cmdln.option('-n', '--name-filter', metavar='EXPR',
-                        help='show only packages whos name matches EXPR')
-                       
+                        help='show only packages whose names match EXPR')
+    @cmdln.option('-p', '--project', metavar='PROJECT',
+                        help='show packages in project PROJECT')
+    
     @cmdln.alias('pr')
     def do_prjresults(self, subcmd, opts, *args):
         """${cmd_name}: Shows project-wide build results
@@ -1495,6 +1616,9 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         2. osc prjresults
                 the project is guessed from the current dir
 
+        3. osc prjresults --project=<project>
+                the project is specified from the command line
+
         ${cmd_usage}
         ${cmd_option_list}
         """
@@ -1503,13 +1627,17 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             print >>sys.stderr, 'getting results for more than one project is not supported'
             return 2
             
-        if args:
-            wd = args[0]
+        if opts.project:
+            project = opts.project
+            apiurl = conf.config['apiurl']
         else:
-            wd = os.curdir
+            if args:
+                wd = args[0]
+            else:
+                wd = os.curdir
 
-        project = store_read_project(wd)
-        apiurl = store_read_apiurl(wd)
+            project = store_read_project(wd)
+            apiurl = store_read_apiurl(wd)
 
         print '\n'.join(get_prj_results(apiurl, project, hide_legend=opts.hide_legend, csv=opts.csv, status_filter=opts.status_filter, name_filter=opts.name_filter))
 
@@ -1521,7 +1649,7 @@ Please submit there instead, or use --nodevelproject to force direct submission.
     @cmdln.option('-s', '--status-filter', metavar='STATUS',
                         help='show only packages with buildstatus STATUS (see legend)')
     @cmdln.option('-n', '--name-filter', metavar='EXPR',
-                        help='show only packages whos name matches EXPR')
+                        help='show only packages whose names match EXPR')
 
     def do_rprjresults(self, subcmd, opts, prj):
         """${cmd_name}: Shows project-wide build results of remote Projects
@@ -1692,6 +1820,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
                   help='don\'t update the package changelog from a changes file')
     @cmdln.option('--noinit', '--no-init', action='store_true',
                   help='Skip initialization of build root and start with build immediately.')
+    @cmdln.option('--nochecks', '--no-checks', action='store_true',
+                  help='Do not run post build checks on the resulting packages.')
     @cmdln.option('--no-verify', action='store_true',
                   help='Skip signature verification of packages used for build.')
     @cmdln.option('-p', '--prefer-pkgs', metavar='DIR', action='append',
@@ -1741,6 +1871,10 @@ Please submit there instead, or use --nodevelproject to force direct submission.
 
         usage: 
             osc build [OPTS] PLATFORM ARCH BUILD_DESCR
+            osc build [OPTS] PLATFORM (ARCH = hostarch, BUILD_DESCR is detected automatically)
+            osc build [OPTS] ARCH (PLATFORM = build_platform (config option), BUILD_DESCR is detected automatically)
+            osc build [OPTS] BUILD_DESCR (PLATFORM = build_platform (config option), ARCH = hostarch)
+            osc build [OPTS] (PLATFORM = build_platform (config option), ARCH = hostarch, BUILD_DESCR is detected automatically)
 
         # Note: 
         # Configuration can be overridden by envvars, e.g.  
@@ -1759,23 +1893,34 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             sys.stderr.write('See http://download.opensuse.org/repositories/openSUSE:/Tools/\n')
             return 1
 
-        if len(args) == 2:
-            raise oscerr.WrongArgs('Missing argument: build description (spec of dsc file)')
-
-        elif len(args) < 2:
-            # we are going to raise an error for this, but first look up some helpful details:
-            msg= ['You have to choose a repo to build on.']
-            msg.append('Possible repositories on this machine are:\n')
-            for platform in get_repos_of_project(store_read_apiurl(os.curdir),
-                                                 store_read_project(os.curdir)):
-                arch = platform.split()[1] # arch
-                if arch == osc.build.hostarch or \
-                   arch in osc.build.can_also_build.get(osc.build.hostarch, []):
-                    msg.append(platform.strip())
-            raise oscerr.WrongArgs('Missing argument.\n\n' + '\n'.join(msg))
-
-        elif len(args) > 3:
+        if len(args) > 3:
             raise oscerr.WrongArgs('Too many arguments')
+        
+        arg_arch = arg_platform = arg_descr = None
+        if len(args) < 3:
+            for arg in args:
+                if arg.endswith('.spec') or arg.endswith('.dsc'):
+                    arg_descr = arg
+                else:
+                    if arg in osc.build.can_also_build.get(osc.build.hostarch, []) or \
+                       arg in osc.build.hostarch:
+                        arg_arch = arg
+                    elif not arg_platform:
+                        arg_platform = arg
+                    else:
+                        raise oscerr.WrongArgs('unexpected argument: \'%s\'' % arg)
+        else:
+            arg_platform, arg_arch, arg_descr = args
+
+        arg_arch = arg_arch or osc.build.hostarch
+        arg_platform = arg_platform or conf.config['build_platform']
+        descr = [ i for i in os.listdir('.') if i.endswith('.spec') or i.endswith('.dsc') ]
+        if not arg_descr and len(descr) == 1:
+            arg_descr = descr[0]
+        elif not arg_descr:
+            raise oscerr.WrongArgs('Missing argument: build description (spec or dsc file)')
+
+        args = (arg_platform, arg_arch, arg_descr)
 
         if opts.prefer_pkgs:
             for d in opts.prefer_pkgs:
@@ -1787,7 +1932,8 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             if not os.path.isdir(opts.keep_pkgs):
                 print >> sys.stderr, 'Preferred save location \'%s\' is not a directory' % opts.keep_pkgs
                 return 1
-
+        
+        print 'Building %s for %s/%s' % (arg_descr, arg_platform, arg_arch)
         return osc.build.main(opts, args)
 
             
@@ -1832,6 +1978,24 @@ Please submit there instead, or use --nodevelproject to force direct submission.
         print '\n'.join(get_commitlog(apiurl, project, package, rev))
 
 
+    @cmdln.option('-r', '--revision', metavar='rev',
+                        help='show log of the specified revision')
+    def do_rlog(self, subcmd, opts, prj, pkg):
+        """${cmd_name}: Shows the commit log of a remote package
+
+        ${cmd_usage}
+        ${cmd_option_list}
+        """
+
+        apiurl = conf.config['apiurl']
+        rev, dummy = parseRevisionOption(opts.revision)
+        if rev and not checkRevision(prj, pkg, rev, apiurl):
+            print >>sys.stderr, 'Revision \'%s\' does not exist' % rev
+            sys.exit(1)
+
+        print '\n'.join(get_commitlog(apiurl, prj, pkg, rev))
+
+
     @cmdln.option('-f', '--failed', action='store_true',
                   help='rebuild all failed packages')
     def do_rebuildpac(self, subcmd, opts, *args):
@@ -1969,6 +2133,58 @@ Please submit there instead, or use --nodevelproject to force direct submission.
             print wipebinaries(conf.config['apiurl'], args[0], package, opts.arch, opts.repo, code)
 
 
+    @cmdln.option('-q', '--quiet', action='store_true',
+                  help='do not show downloading progress')
+    @cmdln.option('-d', '--destdir', default='.', metavar='DIR',
+                  help='destination directory')
+    def do_getbinaries(self, subcmd, opts, project, package, repository, architecture):
+        """${cmd_name}: Download binaries to a local directory
+
+        This command does download packages directly from the api server. 
+        Thus, it does directly access the packages that are used for building
+        others, even when they are "published" yet.
+
+        ${cmd_usage}
+        ${cmd_option_list}
+        """
+
+        # Get package list
+        binaries = get_binarylist(conf.config['apiurl'], 
+                                   project, repository, architecture, 
+                                   package = package, verbose=True)
+
+        if not os.path.isdir(opts.destdir):
+            print "Creating %s" % opts.destdir
+            os.makedirs(opts.destdir, 0755)
+
+        if binaries == [ ]:
+            sys.exit('no binaries found. Either the package does not '
+                     'exist, or no binaries have been built.')
+
+        for binary in binaries:
+
+            # skip source rpms
+            if binary.name.endswith('.src.rpm'):
+                continue
+
+            target_filename = '%s/%s' % (opts.destdir, binary.name)
+
+            if os.path.exists(target_filename):
+                st = os.stat(target_filename)
+                if st.st_mtime == binary.mtime and st.st_size == binary.size:
+                    continue
+
+            get_binary_file(conf.config['apiurl'], 
+                            project, 
+                            repository, architecture, 
+                            binary.name, 
+                            package = package,
+                            target_filename = target_filename,
+                            target_mtime = binary.mtime,
+                            progress_meter = not opts.quiet)
+
+
+
     @cmdln.option('--repos-baseurl', action='store_true',
                         help='show base URLs of download repositories')
     @cmdln.option('-e', '--enable-exact', action='store_true',
