@@ -19,20 +19,22 @@ The configuration dictionary could look like this:
 
 {'apisrv': 'https://api.opensuse.org/',
  'user': 'poeml',
- 'auth_dict': {'api.opensuse.org': {'user': 'poeml', 'pass': 'secret'},
-               'apitest.opensuse.org': {'user': 'poeml', 'pass': 'secret'},
-               'foo.opensuse.org': {'user': 'foo', 'pass': 'foo'}},
+ 'api_host_options': {'api.opensuse.org': {'user': 'poeml', 'pass': 'secret'},
+                      'apitest.opensuse.org': {'user': 'poeml', 'pass': 'secret',
+                                               'http_headers':(('Host','api.suse.de'),
+                                                               ('User','faye'))},
+                      'foo.opensuse.org': {'user': 'foo', 'pass': 'foo'}},
  'build-cmd': '/usr/bin/build',
  'build-root': '/abuild/oscbuild-%(repo)s-%(arch)s',
  'packagecachedir': '/var/cache/osbuild',
  'su-wrapper': 'sudo',
  'urllist': ['http://download.opensuse.org/repositories/%(project)s/%(repository)s/%(arch)s/%(filename)s',
-             'http://api.opensuse.org/rpm/%(project)s/%(repository)s/_repository/%(buildarch)s/%(name)s'],
+             'http://api.opensuse.org/rpm/%(project)s/%(repository)s/%(repopackage)s/%(buildarch)s/%(name)s'],
  }
 
 """
 
-import ConfigParser
+import OscConfigParser
 from osc import oscerr
 
 # being global to this module, this dict can be accessed from outside
@@ -53,7 +55,7 @@ DEFAULTS = { 'apisrv': 'https://api.opensuse.org/',
                 # the normal repo server, redirecting to mirrors
                 'http://download.opensuse.org/repositories/%(project)s/%(repository)s/%(arch)s/%(filename)s',
                 # direct access to "full" tree
-                '%(scheme)s://%(apisrv)s/build/%(project)s/%(repository)s/%(buildarch)s/_repository/%(name)s',
+                '%(scheme)s://%(apisrv)s/build/%(project)s/%(repository)s/%(buildarch)s/%(repopackage)s/%(name)s',
               ],
 
              'debug': '0',
@@ -65,6 +67,8 @@ DEFAULTS = { 'apisrv': 'https://api.opensuse.org/',
              'do_package_tracking': '0',
              # default for osc build
              'extra-pkgs': 'vim gdb strace',
+             # default platform
+             'build_platform': 'openSUSE_Factory',
 }
 boolean_opts = ['debug', 'do_package_tracking', 'http_debug', 'post_mortem', 'traceback']
 
@@ -92,6 +96,9 @@ new_conf_template = """
 # you can have an empty setting here.
 #extra-pkgs = vim gdb strace
 
+# build platform is used if the platform argument is omitted to osc build
+#build_platform = openSUSE_Factory
+
 # show info useful for debugging 
 #debug = 1
     
@@ -107,6 +114,9 @@ new_conf_template = """
 [%(apisrv)s]
 user = %(user)s
 pass = %(pass)s
+# additional headers to pass to a request, e.g. for special authentication
+#http_headers = Host: foofoobar,
+#       User: mumblegack
 """
 
 
@@ -116,7 +126,6 @@ You will be asked for them below, and they will be stored in
 %s for future use.
 """
 
-
 config_incomplete_text = """
 
 Your configuration file %s is not complete.
@@ -125,6 +134,11 @@ Make sure that it has a [general] section.
 
 """
 
+config_missing_apiurl_text = """
+the apiurl \'%s\' does not exist in the config file. Please enter
+your credentials for this apiurl.
+"""
+
 cookiejar = None
 
 def parse_apisrv_url(scheme, apisrv):
@@ -134,18 +148,33 @@ def parse_apisrv_url(scheme, apisrv):
     else:
         return scheme, apisrv
 
+def get_apiurl_api_host_options(apiurl):
+    """
+    Returns all apihost specific options for the given apiurl, None if
+    no such specific optiosn exist.
+    """
+    # FIXME: in A Better World (tm) there was a config object which
+    # knows this instead of having to extract it from a url where it
+    # had been mingled into before.  But this works fine for now.
+    scheme, apisrv = parse_apisrv_url(None, apiurl)
+    return config['api_host_options'][apisrv]
+
 def get_apiurl_usr(apiurl):
     """
     returns the user for this host - if this host does not exist in the
-    internal auth_dict the default user is returned.
+    internal api_host_options the default user is returned.
     """
+    # FIXME: maybe there should be defaults not just for the user but
+    # for all apihost specific options.  The ConfigParser class
+    # actually even does this but for some reason we don't use it
+    # (yet?).
+
     import sys
-    scheme, apisrv = parse_apisrv_url(None, apiurl)
-    if config['auth_dict'].has_key(apisrv):
-        return config['auth_dict'][apisrv]['user']
-    else:
-        print >>sys.stderr, 'section [\'%s\'] does not exist - using default user: \'%s\'' \
-            % (apisrv, config['user'])
+    try:
+        return get_apiurl_api_host_options(apiurl)['user']
+    except KeyError:
+        print >>sys.stderr, 'no specific section found in config file for host of [\'%s\'] - using default user: \'%s\'' \
+            % (apiurl, config['user'])
         return config['user']
 
 def init_basicauth(config):
@@ -194,7 +223,7 @@ def init_basicauth(config):
 
     # with None as first argument, it will always use this username/password
     # combination for urls for which arg2 (apisrv) is a super-url
-    for host, auth in config['auth_dict'].iteritems():
+    for host, auth in config['api_host_options'].iteritems():
         authhandler.add_password(None, host, auth['user'], auth['pass'])
 
 
@@ -208,39 +237,38 @@ def get_configParser(conffile=None, force_read=False):
     conffile = conffile or os.environ.get('OSC_CONFIG', '~/.oscrc')
     conffile = os.path.expanduser(conffile)
     if force_read or not get_configParser.__dict__.has_key('cp'):
-        get_configParser.cp = ConfigParser.SafeConfigParser(DEFAULTS)
+        get_configParser.cp = OscConfigParser.OscConfigParser(DEFAULTS)
         get_configParser.cp.read(conffile)
     return get_configParser.cp
 
 
-def write_config(conffile, entries, custom_template = None, force = False):
+def write_initial_config(conffile, entries, custom_template = ''):
     """
-    write osc's configuration file. entries is a dict which contains values
+    write osc's intial configuration file. entries is a dict which contains values
     for the config file (e.g. { 'user' : 'username', 'pass' : 'password' } ).
-    custom_template is an optional configuration template. Use force=True if you
-    want to overwrite an existing configuration file.
+    custom_template is an optional configuration template.
     """
-    import os
+    import os, StringIO, sys
     conf_template = custom_template or new_conf_template
     config = DEFAULTS.copy()
     config.update(entries)
-    if force or not os.path.exists(conffile):
-        file = None
+    sio = StringIO.StringIO(conf_template.strip() % config)
+    cp = OscConfigParser.OscConfigParser(DEFAULTS)
+    cp.readfp(sio)
+
+    file = None
+    try:
+        file = open(conffile, 'w')
+    except IOError, e:
+        raise oscerr.OscIOError(e, 'cannot open configfile \'%s\'' % conffile)
+    try:
         try:
-            file = open(conffile, 'w')
+            os.chmod(conffile, 0600)
+            cp.write(file, True)
         except IOError, e:
-            raise oscerr.OscIOError(e, 'cannot open configfile \'%s\'' % conffile)
-        try:
-            try:
-                os.chmod(conffile, 0600)
-                file.write(conf_template % config)
-            except IOError, e:
-                raise oscerr.OscIOError(e, 'cannot write configfile \'s\'' % conffile)
-        finally:
-            if file: file.close()
-        return True
-    else:
-        return False
+            raise oscerr.OscIOError(e, 'cannot write configfile \'s\'' % conffile)
+    finally:
+        if file: file.close()
 
 
 def get_config(override_conffile = None, 
@@ -252,33 +280,15 @@ def get_config(override_conffile = None,
     """do the actual work (see module documentation)"""
     import os
     import sys
+    import re
     global config
 
     conffile = override_conffile or os.environ.get('OSC_CONFIG', '~/.oscrc')
     conffile = os.path.expanduser(conffile)
 
     if not os.path.exists(conffile):
-
-        # okay, let's create a fresh config file
-        # if credentials are found in .netrc, use those
-        # otherwise ask
-
-        config = DEFAULTS.copy()
-
-        # try .netrc
-        # the needed entry needs to look like this:
-        # machine api.opensuse.org login your_login password your_pass
-        # note that it is not suited for credentials containing spaces
-        import netrc
-        try:
-            # XXX: apisrv is a URL now, thus requiring the "scheme" setting if https is to be used
-            netrc_host = parse_apisrv_url(None, DEFAULTS['apisrv'])[1]
-            config['user'], account, config['pass'] = \
-                    netrc.netrc().authenticators(netrc_host)
-            print >>sys.stderr, 'Read credentials from %s.' % os.path.expanduser('~/.netrc')
-        except (IOError, TypeError, netrc.NetrcParseError):
-            raise oscerr.NoConfigfile(conffile, \
-                                              account_not_configured_text % conffile)
+        raise oscerr.NoConfigfile(conffile, \
+                                  account_not_configured_text % conffile)
 
     # okay, we made sure that .oscrc exists
 
@@ -288,7 +298,7 @@ def get_config(override_conffile = None,
         # FIXME: it might be sufficient to just assume defaults?
         msg = config_incomplete_text % conffile
         msg += new_conf_template % DEFAULTS
-        raise oscerr.ConfigError(msg)
+        raise oscerr.ConfigError(msg, conffile)
 
     config = dict(cp.items('general', raw=1))
 
@@ -297,33 +307,51 @@ def get_config(override_conffile = None,
 
     for i in boolean_opts:
         try:
-            if int(config.get(i)):
-                config[i] = True
-            else:
-                config[i] = False
-        except:
-            raise oscerr.ConfigError('option %s requires an integer value' % i)
+            config[i] = cp.getboolean('general', i)
+        except ValueError, e:
+            raise oscerr.ConfigError('cannot parse \'%s\' setting: ' % i + str(e), conffile)
 
     config['packagecachedir'] = os.path.expanduser(config['packagecachedir'])
 
-    config['extra-pkgs'] = [ i.strip(',') for i in config['extra-pkgs'].split() ]
+    re_clist = re.compile('[, ]+')
+    config['extra-pkgs'] = [ i.strip() for i in re_clist.split(config['extra-pkgs'].strip()) if i ]
     if config['extra-pkgs'] == []: 
         config['extra-pkgs'] = None
 
     # transform 'url1, url2, url3' form into a list
     if type(config['urllist']) == str:
-        config['urllist'] = [ i.strip() for i in config['urllist'].split(',') ]
+        config['urllist'] = [ i.strip() for i in re_clist.split(config['urllist'].strip()) ]
+
+    # collect the usernames, passwords and additional options for each api host
+    api_host_options = { }
+
+    # Regexp to split extra http headers into a dictionary
+    # the text to be matched looks essentially looks this:
+    # "Attribute1: value1, Attribute2: value2, ..."
+    # there may be arbitray leading and intermitting whitespace.
+    # the following regexp does _not_ support quoted commas within the value.
+    http_header_regexp = re.compile(r"\s*(.*?)\s*:\s*(.*?)\s*(?:,\s*|\Z)")
 
-    # holds multiple usernames and passwords
-    auth_dict = { } 
     for url in [ x for x in cp.sections() if x != 'general' ]:
         dummy, host = \
             parse_apisrv_url(config['scheme'], url)
-        auth_dict[host] = { 'user': cp.get(url, 'user'), 
-                            'pass': cp.get(url, 'pass') }
+        #FIXME: this could actually be the ideal spot to take defaults
+        #from the general section.
+        user         = cp.get(url, 'user')
+        password     = cp.get(url, 'pass')
+
+        if cp.has_option(url, 'http_headers'):
+            http_headers = cp.get(url, 'http_headers')
+            http_headers = http_header_regexp.findall(http_headers)
+        else:
+            http_headers = []
+
+        api_host_options[host] = { 'user': user,
+                                   'pass': password,
+                                   'http_headers': http_headers};
 
     # add the auth data we collected to the config dict
-    config['auth_dict'] = auth_dict
+    config['api_host_options'] = api_host_options
 
     # override values which we were called with
     if override_debug: 
@@ -344,8 +372,11 @@ def get_config(override_conffile = None,
     # XXX unless config['user'] goes away (and is replaced with a handy function, or 
     # config becomes an object, even better), set the global 'user' here as well,
     # provided that there _are_ credentials for the chosen apisrv:
-    if config['apisrv'] in config['auth_dict'].keys():
-        config['user'] = config['auth_dict'][config['apisrv']]['user']
+    if config['apisrv'] in config['api_host_options'].keys():
+        config['user'] = config['api_host_options'][config['apisrv']]['user']
+    else:
+        raise oscerr.ConfigMissingApiurl(config_missing_apiurl_text % config['apisrv'],
+                                         conffile, config['apiurl'])
 
     # finally, initialize urllib2 for to use the credentials for Basic Authentication
     init_basicauth(config)
