@@ -23,7 +23,7 @@ def join_url(self, base_url, rel_url):
 
 
 class Fetcher:
-    def __init__(self, cachedir = '/tmp', auth_dict = {}, urllist = [], http_debug = False):
+    def __init__(self, cachedir = '/tmp', api_host_options = {}, urllist = [], http_debug = False):
 
         __version__ = '0.1'
         __user_agent__ = 'osbuild/%s' % __version__
@@ -40,8 +40,8 @@ class Fetcher:
         self.http_debug = http_debug
 
         passmgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
-        for host in auth_dict.keys():
-            passmgr.add_password(None, host, auth_dict[host]['user'], auth_dict[host]['pass'])
+        for host in api_host_options.keys():
+            passmgr.add_password(None, host, api_host_options[host]['user'], api_host_options[host]['pass'])
         authhandler = urllib2.HTTPBasicAuthHandler(passmgr)
         self.gr = URLGrabber(user_agent=__user_agent__,
                             keepalive=1,
@@ -134,26 +134,25 @@ def verify_pacs(pac_list):
        Check all packages in one go, since this takes only 6 seconds on my Athlon 700
        instead of 20 when calling 'rpm -K' for each of them.
        """
-
+    import subprocess
 
     if not pac_list:
         return
         
-    # we can use os.popen4 because we don't care about the return value.
-    # we check the output anyway, and rpm always writes to stdout.
+    # don't care about the return value because we check the
+    # output anyway, and rpm always writes to stdout.
 
     # save locale first (we rely on English rpm output here)
     saved_LC_ALL = os.environ.get('LC_ALL')
     os.environ['LC_ALL'] = 'en_EN'
 
-    (i, o) = os.popen4(['/bin/rpm', '-K'] + pac_list)
+    o = subprocess.Popen(['/bin/rpm', '-K'] + pac_list, stdout=subprocess.PIPE,
+                stderr=subprocess.STDOUT, close_fds=True).stdout
 
     # restore locale
     if saved_LC_ALL: os.environ['LC_ALL'] = saved_LC_ALL;
     else: os.environ.pop('LC_ALL')
 
-    i.close()
-
     for line in o.readlines():
 
         if not 'OK' in line:
